package XML::WordML::SimpleWriter;

=head1 XML::WordML::SimpleWriter


=head1 NAME

	Xml::WordML::SimpleWrite - Write simple Ms Word XML files

=head1 SYNOPSIS

	use XML::WordML::SimpleWriter;

	my $wml = new XML::WordML::SimpleWriter;

	# document settings
   	$wml->set_encoding('LATIN1');
   	$wml->set_defaultFont('Arial');
   	$wml->load_stylesheet('style.xml');


	# title 1
   	$wml->begin_subsection(); # subsection begins before title
  	$wml->append_heading(1,"This is a Title 1");


	# title 2
	$wml->begin_subsection();
	$wml->append_heading(2, "This is a Title 2");	


	# some paragraphs
        $wml->append_parag("Lorem ipsum and so on");
	
	$wml->begin_parag();
	$wml->append("Lorem ipsum and so on .... encore");
	$wml->end_param();


	# lists
        $wml->begin_subsection();
	$wml->begin_listitem($lvl);
        $wml->append("item ");
	$wml->append("one");
	$wml->end_list_item();
        $wml->append_listitem($lvl,"item two");
	$wml->end_subsection();


	# a  4 x 3 table
	$wml->begin_table('StdTable');
	for( my $row =0; $row < 4; $row++ ) { 
            $wml->begin_tablerow('head'); 
		for( my $col =0; $col < 3; $col++ ) {
            		$wml->append_tablecell("this is cell col:$col,row:$row");
		}
            $wml->end_tablerow();
	$wml->end_table();


	$wml->end_subsection(); # closing title 2 subsection
	$wml->end_subsection(); # closing title 1 subsection

	print $wml->xml();

=head1 DESCRIPTION

=head2 Classes methods

=head3 new()

Create a new instance of the class

=head3 default_stylesheet() 

Returns the default stylesheet

=head2 Document properties

=head3 properties(\%pr) 

Sets the properties from a hash table

=head3 set_defaultFont($font_name) 

Sets the default font to $font_name.

=head3 set_encoding($enc)

Sets the text encoding to $enc .

=head3 title($ttle) 

Sets the document title.

=head3 author($name)

Sets the document author field.

=head2 Styles

=head3 load_stylesheet($fname)

Loads an external spreadsheet from the $fname file

=head3 set_stylesheet($stylesheet)

Sets the current stylesheet to the content of $stylesheet

=head3 stylesheet()

Gets the current stylesheet.

=head2 Headings and paragraphes

=head3 begin_heading($lvl)

Start a title of level $lvl.

=head3 begin_parag([style])

Start a paragraph of the named style .

Default style is used if none precised.

=head3 enf_parag()

Finishes a paragraph section.

=head3 begin_subsection()

Starts a subsection (text bloc).

=head3 end_subsection()

End a subsection.

=head3 append(text,[option])

Appends text to the current paragraph.

Second parameter (optional) is a string style:
i for italic, b for bold, u for underline 
or combination of them

=head2 Tables manipulation

=head3 begin_table([style,[cols,[rows]]])

Start a table of cols columns and rows rows with the given style.

=head3 end_table()

Finishes a table section

=head3 begin_tablerow([is_head])

Start a table row.

If is_head is true, the row is to be considered a table header

=head3 end_tablerow()

Finishes a table row.

=head3 begin_tablecell()

Starts a table cell.

=head3 end_tablecell()

Finishes a table cell.

=head2 Lists and enumerations

=head3 begin_listitem([lvl])

Starts an enumeration section.

=head3 end_listitem()

Finishes an enumeration section.

=head1 KNOWN PROBLEMS

none for now, it wont last.

=head1 SEE ALSO

nothing now

=head1 COPYRIGHT

Copyright (c) 2003 - 2006 Christophe RENARD All rights reserved

You may distribute under the terms of either the GNU Lesser General Public License or the Artistic License, as specified in the Perl README file.

=cut


use strict;

use Unicode::String qw(utf8 latin1 utf16);

sub new 
{ 
    my $this = shift;
    my $class = ref($this) || $this;
    $this = {};
    bless $this, $class;
    $this->initialize();
    return $this;
}#eo new

sub save_curnode 
{
   my ($this) =@_;
   my $rPs = $this->{'PARAGRAPHS'};
   if( defined( $this->{'CURNODE'}  ) ) {
      push(@$rPs, $this->{'CURNODE'} );
      $this->{'CURNODE'} = undef;
   }
}

sub begin_heading($$) 
{
   my ($this,$lvl) =@_;
   $this->begin_parag("Titre$lvl");
   #$this->begin_parag("Heading$lvl");
}#eo begin_heading


sub begin_parag
{ 
   my ($this,$style) =@_;
   $style ='Normal' if( !defined($style) );
   $this->add_parag( { 'pStyle' => { 'val' => $style }  } );
}#eo begin_parag


sub begin_table($$$$)
{
   my ($this,$style,$cols,$rows) = @_;
    $this->add_tablehead( {
        'tblStyle' => { 'val' => $style }, 
	'tblW'    => { 'w'   => 0, 'type' => 'auto' } ,
	'tblLook' => { 'val' => '000001E0' }
    } );   
}#eo begin_table


sub begin_tablerow  
{ 
    my ($this,$ishead)=@_; 
    if( defined($ishead) and $ishead ) {
	$this->add_tablerow( { 'tblHeader' => { 'val' => 'onOffType' } } )
    } else {
    	$this->add_tablerow();   
    }
}#eo begin_tablerow

sub end_table($)        { my ($this)=@_; $this->add_node('TABLE-END');	     }
sub end_tablerow($)     { my ($this)=@_; $this->add_node('TABLEROW-END');     }
sub begin_tablecell($)  { my ($this)=@_; $this->add_node('TABLECELL-BEGIN');  }
sub end_tablecell($)    { my ($this)=@_; $this->add_node('TABLECELL-END');    }
sub begin_subsection($) { my ($this)=@_; $this->add_node('SUBSECTION-BEGIN'); }
sub end_subsection($)   { my ($this)=@_; $this->add_node('SUBSECTION-END');   }

sub begin_listitem
{
  my ($this,$lvl) = @_;
    $this->add_parag( { 'ilvl' => { 'val' =>$lvl }, 'ilfo'=> { 'val' =>2 } } );  
}#eo begin_li

sub end_listitem
{
    my ($this) = @_;
    $this->end_parag();	
}#eo end li

sub end_parag
{
   my ($this) =@_;
   $this->save_curnode();  
}#eo end_parag

# append text to the current paragraph
# second parameter (optional) is style:
# i for italic
# b for bold
# u for underline
# or combination of them
sub append 
{
   my ($this,$xml,$style,$color) =@_;
   my $rPr = {};
   if( defined( $style ) )  {
      if( $style =~ m/i/ ) { $rPr->{'i'}=''; }
      if( $style =~ m/b/ ) { $rPr->{'b'}=''; }
      if( $style =~ m/u/ ) { $rPr->{'u'} = { 'val'=>"single" };  }
      <w:color w:val="FF0000"/>
   }
   if( defined( $color ) ) {
      $rPr->{'color'} = { 'val'=>$color };
   }
   $this->add_text($xml,$rPr);
}#eo append

sub append_heading($$$) 
{
   my ($this,$lvl,$text) = @_;
   $this->begin_heading($lvl);
   $this->append($text);
   $this->end_parag();
}#eo append_heading

sub append_listitem($$$)
{
   my ($this,$lvl,$text) = @_;
   $this->begin_listitem($lvl);
   $this->append($text);
   $this->end_parag();
}#eo append_listitem


sub append_parag($$$) 
{
   my ($this,$style,$text) = @_;
   $this->begin_parag($style);
   $this->append($text);
   $this->end_parag();
}#eo append_parag

sub append_tablecell($$)
{
   my ($this,$text) = @_;
   $this->begin_tablecell();
   $this->append_parag('Normal',$text);
   $this->end_tablecell();
}#eo append tabelcell

sub append_styled_tablecell($$$)
{
   my ($this,$text,$style) = @_;
   $this->begin_tablecell($style);
   $this->begin_parag('Normal');
   $this->append($text,$style);
   $this->end_parag();
   $this->end_tablecell();
}#eo append styled tablecell

sub add_node($$)
{
    my ($this,$type) = @_;

    my $rPs = $this->{'PARAGRAPHS'};
    $this->save_curnode();
    my $node = { 'TYPE'=>$type }; 
    push(@$rPs, $node);
    return $node
}#eo add_node

sub add_parag($$)
{
   my ($this,$rPr) =@_;
   my $new = {};
   $new->{'PROPERTIES'}= $rPr;
   $new->{'CONTENT'}   = [];
   $new->{'TYPE'}      = 'P';
   $this->save_curnode();
   $this->{'CURNODE'}  = $new;
   $this->{'DIRTY'}=1
}#eo add_parag

sub add_tablehead($$)
{
  my ($this,$rPr) = @_;
  my $new = {};
  $new->{'PROPERTIES'}= $rPr;
  $new->{'CONTENT'}   = [];
  $new->{'TYPE'}      = 'TABLE-BEGIN';
  $this->save_curnode();
  $this->{'CURNODE'}  = $new;
  $this->{'DIRTY'}=1
}#eo add_tablehead

sub add_tablerow($$)
{
  my ($this,$rPr) = @_;
  my $new = {};
  $new->{'PROPERTIES'}= $rPr;
  $new->{'CONTENT'}   = [];
  $new->{'TYPE'}      = 'TABLEROW-BEGIN';
  $this->save_curnode();
  $this->{'CURNODE'}  = $new;
  $this->{'DIRTY'}=1
}#eo add_tablehead

sub add_text($$$) 
{
   my ($this,$xml,$rPr) =@_;
   my $node;
   
   $this->add_parag({}) if( !defined( $this->{'CURNODE'} ) );
   
   $node = $this->{'CURNODE'};
   
   my $new = {};
   $new->{'PROPERTIES'}=$rPr;
   $new->{'CONTENT'} = $xml;
   $new->{'TYPE'}    = 'R';

   my $rCnt = $node->{'CONTENT'};
   push(@$rCnt,$new);
   
   $this->{'CURNODE'} = $node;
   $this->{'DIRTY'} =1;

}#eo add_text

sub generatePrTags($$$)
{
    my ($this,$tpe,$rPr) = @_;
    my $xml='';
    my @prs = keys %$rPr;
    if ( scalar(@prs)>0 ) {
         $xml .= "\t<w:".$tpe."Pr>\n";
         foreach my $tag ( @prs ) {
            my $val = $rPr->{$tag};
            if( ref($val) eq 'HASH' ) {
               my $attrs='';
               foreach my $attr ( keys %$val ) {
		  next unless defined($val->{$attr});
                  $attrs .= "w:$attr='".$val->{$attr}."' ";
               }
               $xml .= "\t\t<w:$tag $attrs />\n"
            } else {
               $xml .= "\t\t<w:$tag>$val</w:$tag>\n";
            }
         }
	 $xml .= "\t</w:".$tpe."Pr>\n";
    }
    return $xml;
}#eo generatePrTags


sub generate_tablehead($$) 
{
   my ($this,$node) = @_;
   my $xml="<w:tbl>\n";
   if ( defined( $node->{'PROPERTIES'} ) ) {
      my $rPr = $node->{'PROPERTIES'};
      $xml .= $this->generatePrTags('tbl',$rPr);
   }

   return $xml;
}#eo generate_table

sub generate_tablerow_begin($$) 
{
   my ($this,$node) = @_;
   my $xml="<w:tr>\n";
   if ( defined( $node->{'PROPERTIES'} ) ) {
      my $rPr = $node->{'PROPERTIES'};
      $xml .= $this->generatePrTags('tr',$rPr);
   }
   return $xml;
}#eo generate_table

sub generate_parag($$) 
{
   my ($this,$node) = @_;
   my $xml="<w:p>\n";
   if ( defined( $node->{'PROPERTIES'} ) ) {
      my $rPr = $node->{'PROPERTIES'};
      $xml .= $this->generatePrTags('p',$rPr);
   }
   my $blocks = $node->{'CONTENT'};
   foreach my $block ( @$blocks ) {
      $xml.=$this->generate_text($block);
   }
   $xml .= "</w:p>\n";
   return $xml;
}#eo generate_parag

sub generate_text($$) 
{
   my ($this,$node) = @_;
   
   #print $node->{'CONTENT'}."\n";

   my $xml="<w:r>";
   if ( defined( $node->{'PROPERTIES'} )) {
      my $rPr = $node->{'PROPERTIES'};
      $xml .= $this->generatePrTags('r',$rPr);
   }
   if( defined($node->{'CONTENT'}) ) {
	$xml .= "<w:t>".$node->{'CONTENT'}."</w:t>";
   }
   $xml .= "</w:r>\n";
   return $xml;
}#eo generate_text

sub generate_docProperties($)
{
   my ($this) =@_;
   my $xml;
   my $rProps = $this->{'PROPERTIES'};
   $xml = "<o:DocumentProperties>\n";
   foreach my $tag (keys %$rProps) {
      my $val = $rProps->{$tag};
      $xml .= "\t<o:$tag>$val</o:$tag>\n"
   }
   $xml .= "</o:DocumentProperties>\n";
   return $xml;
}#eo generatePropertiesText

sub generate_body($) {
   my ($this,$root) =@_;
   my $xml='';
   $this->save_curnode();
   my $parags = $this->{'PARAGRAPHS'};
   foreach my $parag ( @$parags ) {
      if(    $parag->{'TYPE'} eq 'P'                 ) { $xml .= $this->generate_parag($parag);            } 
      elsif( $parag->{'TYPE'} eq 'TABLE-BEGIN'       ) { $xml .= $this->generate_tablehead($parag);        } 
      elsif( $parag->{'TYPE'} eq 'TABLE-END'         ) { $xml .="</w:tbl>\n";                              } 
      elsif( $parag->{'TYPE'} eq 'TABLEROW-BEGIN'    ) { $xml .= $this->generate_tablerow_begin($parag);   } 
      elsif( $parag->{'TYPE'} eq 'TABLEROW-END'      ) { $xml .="</w:tr>\n";                               } 
      elsif( $parag->{'TYPE'} eq 'TABLECELL-BEGIN'   ) { $xml .="<w:tc>\n";                                } 
      elsif( $parag->{'TYPE'} eq 'TABLECELL-END'     ) { $xml .="</w:tc>\n";                               }
      elsif( $parag->{'TYPE'} eq 'SUBSECTION-BEGIN'  ) { $xml .="<wx:sub-section>\n";                      } 
      elsif( $parag->{'TYPE'} eq 'SUBSECTION-END'    ) { $xml .="</wx:sub-section>\n";                     }
   }
   return $xml;
}#eo generate_body

sub generate_xml($) 
{
   my ($this) =@_;
   my $xml;
   $xml  = $this->document_prefix(); 
   $xml .= $this->generate_docProperties();
   if( !defined($this->{'STYLESHEET'}) ) {
      $xml .= $this->default_stylesheet();
   } else {
      $xml .= $this->stylesheet();
   }
   $xml .= "<w:body>\n";
   $xml .= $this->generate_body( $this->{'BODY'} );
   $xml .= "</w:body>\n";
   $xml .= $this->document_postfix();
   return $xml;
}#eo generate xml

sub xml($) 
{
   my ($this) =@_;
   return $this->as_string();
}#eo xml

sub as_string($) 
{
   my ($this) =@_;
   if( $this->{'DIRTY'} ) {
      $this->{'XML'} = $this->generate_xml();
      $this->{'DIRTY'}=0;
   }
   return $this->{'XML'};
}#eo as_string

sub initialize($) 
{
   my ($this) = @_;
   $this->{'XML'}  = '';        
   $this->{'BODY'}  = [];
   $this->{'DIRTY'} = 0;
   $this->{'CURNODE'}=undef;
   $this->{'PROPERTIES'}={};
   $this->{'PARAGRAPHS'}=[];
   $this->{'STYLESHEET'}=undef;
}#eo initialize

sub title($$) 
{
   my ($this, $title) = @_; $this->properties({'title' => $title });
}#eo title

sub author($$) 
{
   my ($this, $author) = @_;  $this->properties({'author' => $author });
}#eo author

sub company($$) 
{
   my ($this, $company) = @_;  $this->properties({'company' => $company });
}#eo company

sub properties($$) 
{  
     my ($this, $rProps) = @_;

     foreach my $key ( keys %$rProps ) {
        $this->{'PROPERTIES'}->{ ucfirst(lc($key)) } = $rProps->{$key};
     }
}#eo properties

sub document_prefix() {
	my $this =shift;
	my $enc=  ($this->{'ENCODING'})?$this->{'ENCODING'}:'UTF8';
return <<EOXML;
<?xml version="1.0" encoding="$enc" standalone="yes"?>
<?mso-application progid="Word.Document"?>
<w:wordDocument xmlns:w="http://schemas.microsoft.com/office/word/2003/wordml" xmlns:v="urn:schemas-microsoft-com:vml" xmlns:w10="urn:schemas-microsoft-com:office:word" xmlns:sl="http://schemas.microsoft.com/schemaLibrary/2003/core" xmlns:aml="http://schemas.microsoft.com/aml/2001/core" xmlns:wx="http://schemas.microsoft.com/office/word/2003/auxHint" xmlns:o="urn:schemas-microsoft-com:office:office" xmlns:dt="uuid:C2F41010-65B3-11d1-A29F-00AA00C14882" w:macrosPresent="no" w:embeddedObjPresent="no" w:ocxPresent="no" xml:space="preserve">
EOXML
}

sub document_postfix() 
{
   return <<EOXML;
   </w:wordDocument>
EOXML
}

sub set_encoding($$)
{
	my ($this,$enc) = @_;
	$this->{'ENCODING'} = $enc;
}


sub set_defaultFont($$) 
{
   my ($this,$font) = @_;
   $this->{'DEFAULT_FONT'} = $font;
}#eo set_defaultFont

sub set_stylesheet($$) 
{
   my ($this,$sheet) = @_;
   $this->{'STYLESHEET'} = $sheet;
}

sub load_stylesheet($$) 
{
   my ($this,$fname) = @_;
   if( open FSTYLE,$fname) { 
      my $xml='';
      while( my $line =<FSTYLE> ) {
         #my $l1 = latin1($line);
         #$xml .= $l1->utf8();
         $xml .= $line;
      }
      close FSTYLE;
      $this->{'STYLESHEET'} = $xml;
   } else{
       warn "could not open stylesheet '$fname' : $!";
   }
}


sub stylesheet($) 
{
   my ($this) = @_;
   my $font = $this->{'DEFAULT_FONT'};
   my $xml ="<w:fonts><w:defaultFonts w:ascii='$font' w:fareast='$font' w:h-ansi='$font' w:cs='$font'/></w:fonts>";
   $xml .= $this->{'STYLESHEET'};
   return $xml;
}#eo stylesheet

sub default_stylesheet($) 
{
   my ($this) = @_;
   my $font = $this->{'DEFAULT_FONT'};
   my $xml= <<EOXML;
<w:styles>
        <w:versionOfBuiltInStylenames w:val="4"/>
        <w:latentStyles w:defLockedState="off" w:latentStyleCount="156"/>
        <w:style w:type="paragraph" w:default="on" w:styleId="Normal">
            <w:name w:val="Normal"/>
            <w:rsid w:val="006F6074"/>
            <w:rPr>
                <wx:font wx:val="lveticaNeue-Condensed"/>
                <w:lang w:val="FR" w:fareast="FR" w:bidi="AR-SA"/>
                <w:sz w:val="18"/>
                <w:sz w:val="18"/>
            </w:rPr>
        </w:style>
        <w:style w:type="paragraph" w:styleId="Titre1">
            <w:name w:val="heading 1"/>
            <wx:uiName wx:val="Titre 1"/>
            <w:basedOn w:val="Normal"/>
            <w:next w:val="Normal"/>
            <w:rsid w:val="006F6074"/>
            <w:pPr>
                <w:pStyle w:val="Titre1"/>
                <w:keepNext/>
                <w:spacing w:before="240" w:after="60"/>
                <w:outlineLvl w:val="0"/>
            </w:pPr>
            <w:rPr>
                <w:rFonts w:ascii="$font" w:h-ansi="$font" w:cs="$font"/>
                <wx:font wx:val="$font"/>
                <w:b/>
                <w:b-cs/>
                <w:kern w:val="32"/>
                <w:sz w:val="32"/>
                <w:sz-cs w:val="32"/>
            </w:rPr>
        </w:style>
        <w:style w:type="paragraph" w:styleId="Titre2">
            <w:name w:val="heading 2"/>
            <wx:uiName wx:val="Titre 2"/>
            <w:basedOn w:val="Normal"/>
            <w:next w:val="Normal"/>
            <w:rsid w:val="006F6074"/>
            <w:pPr>
                <w:pStyle w:val="Titre2"/>
                <w:keepNext/>
                <w:spacing w:before="240" w:after="60"/>
                <w:outlineLvl w:val="1"/>
            </w:pPr>
            <w:rPr>
                <w:rFonts w:ascii="$font" w:h-ansi="$font" w:cs="$font"/>
                <wx:font wx:val="$font"/>
                <w:b/>
                <w:b-cs/>
                <w:i/>
                <w:i-cs/>
                <w:sz w:val="28"/>
                <w:sz-cs w:val="28"/>
            </w:rPr>
        </w:style>
        <w:style w:type="paragraph" w:styleId="Titre3">
            <w:name w:val="heading 3"/>
            <wx:uiName wx:val="Titre 3"/>
            <w:basedOn w:val="Normal"/>
            <w:next w:val="Normal"/>
            <w:rsid w:val="006060AF"/>
            <w:pPr>
                <w:pStyle w:val="Titre3"/>
                <w:keepNext/>
                <w:spacing w:before="240" w:after="60"/>
                <w:outlineLvl w:val="2"/>
            </w:pPr>
            <w:rPr>
                <w:rFonts w:ascii="$font" w:h-ansi="$font" w:cs="$font"/>
                <wx:font wx:val="$font"/>
                <w:b/>
                <w:b-cs/>
                <w:sz w:val="24"/>
                <w:sz-cs w:val="24"/>
            </w:rPr>
        </w:style>
        <w:style w:type="paragraph" w:styleId="Titre4">
            <w:name w:val="heading 4"/>
            <wx:uiName wx:val="Titre 4"/>
            <w:basedOn w:val="Normal"/>
            <w:next w:val="Normal"/>
            <w:rsid w:val="006060AF"/>
            <w:pPr>
                <w:pStyle w:val="Titre4"/>
                <w:keepNext/>
                <w:spacing w:before="240" w:after="60"/>
                <w:outlineLvl w:val="3"/>
            </w:pPr>
            <w:rPr>
                <wx:font wx:val="Times New Roman"/>
                <w:b/>
                <w:b-cs/>
                <w:sz w:val="22"/>
                <w:sz-cs w:val="22"/>
            </w:rPr>
        </w:style>
        <w:style w:type="paragraph" w:styleId="Titre5">
            <w:name w:val="heading 5"/>
            <wx:uiName wx:val="Titre 5"/>
            <w:basedOn w:val="Normal"/>
            <w:next w:val="Normal"/>
            <w:rsid w:val="006060AF"/>
            <w:pPr>
                <w:pStyle w:val="Titre5"/>
                <w:spacing w:before="240" w:after="60"/>
                <w:outlineLvl w:val="4"/>
            </w:pPr>
            <w:rPr>
                <wx:font wx:val="Times New Roman"/>
                <w:b/>
                <w:b-cs/>
                <w:i/>
                <w:i-cs/>
                <w:sz w:val="20"/>
                <w:sz-cs w:val="20"/>
            </w:rPr>
        </w:style>
        <w:style w:type="character" w:default="on" w:styleId="Policepardfaut">
            <w:name w:val="Default Paragraph Font"/>
            <wx:uiName wx:val="Police par défaut"/>
            <w:semiHidden/>
        </w:style>
        <w:style w:type="table" w:default="on" w:styleId="TableauNormal">
            <w:name w:val="Normal Table"/>
            <wx:uiName wx:val="Tableau Normal"/>
            <w:semiHidden/>
            <w:rPr>
                <wx:font wx:val="Times New Roman"/>
            </w:rPr>
            <w:tblPr>
                <w:tblInd w:w="0" w:type="dxa"/>
                <w:tblCellMar>
                    <w:top w:w="0" w:type="dxa"/>
                    <w:left w:w="108" w:type="dxa"/>
                    <w:bottom w:w="0" w:type="dxa"/>
                    <w:right w:w="108" w:type="dxa"/>
                </w:tblCellMar>
            </w:tblPr>
        </w:style>
        <w:style w:type="list" w:default="on" w:styleId="Aucuneliste">
            <w:name w:val="No List"/>
            <wx:uiName wx:val="Aucune liste"/>
            <w:semiHidden/>
        </w:style>
        <w:style w:type="paragraph" w:styleId="Listepuces">
            <w:name w:val="List Bullet"/>
            <wx:uiName wx:val="Liste à puces"/>
            <w:basedOn w:val="Normal"/>
            <w:rsid w:val="006060AF"/>
            <w:pPr>
                <w:pStyle w:val="Listepuces"/>
                <w:listPr>
                    <w:ilfo w:val="6"/>
                </w:listPr>
            </w:pPr>
            <w:rPr>
                <wx:font wx:val="Times New Roman"/>
                <w:sz w:val="18"/>
                <w:sz w:val="18"/>
            </w:rPr>
        </w:style>
        <w:style w:type="paragraph" w:styleId="Listepuces2">
            <w:name w:val="List Bullet 2"/>
            <wx:uiName wx:val="Liste à puces 2"/>
            <w:basedOn w:val="Normal"/>
            <w:rsid w:val="006060AF"/>
            <w:pPr>
                <w:pStyle w:val="Listepuces2"/>
                <w:listPr>
                    <w:ilfo w:val="7"/>
                </w:listPr>
            </w:pPr>
            <w:rPr>
                <wx:font wx:val="$font"/>
                <w:sz w:val="18"/>
                <w:sz w:val="18"/>
            </w:rPr>
        </w:style>
        <w:style w:type="paragraph" w:styleId="Listenumros">
            <w:name w:val="List Number"/>
            <wx:uiName wx:val="Liste à numéros"/>
            <w:basedOn w:val="Normal"/>
            <w:rsid w:val="006060AF"/>
            <w:pPr>
                <w:pStyle w:val="Listenumros"/>
                <w:listPr>
                    <w:ilfo w:val="1"/>
                </w:listPr>
            </w:pPr>
            <w:rPr>
                <wx:font wx:val="$font"/>
            </w:rPr>
        </w:style>
    </w:styles>
EOXML
    return $xml;
}#eo default_properties


1;
#eof

__DATA__

sub add_node($$)
sub add_parag($$)
sub add_text($$$) 

sub append 
sub append_heading($$$) 
sub append_listitem($$$)
sub append_parag($$$) 
sub as_string($) 

sub author($$) 
sub begin_heading($$) 
sub begin_listitem
sub begin_parag
sub begin_subsection($)
sub company($$) 
sub default_stylesheet($) 

sub end_listitem

sub end_parag
sub end_subsection($)

sub begin_table($$$$)
sub end_table($)     
sub begin_tablerow  
sub end_tablerow($)   
sub begin_tablecell($)
sub append_styled_tablecell($$$)
sub append_tablecell($$)



